/*
 * Copyright (c) 2006-2008, Marcus von Appen
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _PAPI_ATK_H_
#define _PAPI_ATK_H_

#include <Python.h>
#include <gmodule.h>
#include <atk/atk.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Interface types used by the AtkObject resolution system to create
 * interface enabled GTypes on the fly.
 * See the GType related function in papi_atkobject.c
 * papi_constants.c contains the names for exposing the constants.
 */
typedef enum
{
    IFACE_INVALID        = 0,       /* Invalid interface identifier. */
    IFACE_ACTION         = 1 << 0,  /* Identifies the AtkAction interface. */
    IFACE_COMPONENT      = 1 << 1,  /* Identifies the AtkComponent interface. */
    IFACE_DOCUMENT       = 1 << 2,  /* Identifies the AtkDocument interface. */
    IFACE_EDITABLE_TEXT  = 1 << 3,  /* Identifies the AtkEditableText
                                     * interface. */
    IFACE_HYPERTEXT      = 1 << 4,  /* Identifies the AtkHypertext interface. */
    IFACE_IMAGE          = 1 << 5,  /* Identifies the AtkImage interface. */
    IFACE_SELECTION      = 1 << 6,  /* Identifies the AtkSelection interface. */
    IFACE_STREAMABLE     = 1 << 7,  /* Identifies the AtkStreamableContent
                                     * interface. */
    IFACE_TABLE          = 1 << 8,  /* Identifies the AtkTable interface. */
    IFACE_TEXT           = 1 << 9,  /* Identifies the AtkText interface. */
    IFACE_VALUE          = 1 << 10, /* Identifies the AtkValue interface. */
    IFACE_HYPERLINK_IMPL = 1 << 11  /* Identifies the AtkHyperlinkImpl
                                    * interface. */
} AtkInterfaceType;

/**
 * PyObject type, that encapsulates an AtkObject.
 */
typedef struct
{
    PyObject_HEAD
    AtkObject *obj;
    PyObject *dict;
    PyObject *prop_handlers;
    PyObject *weakrefs;
} PyAtkObject;
#define ATKOBJECT(x) (ATK_OBJECT (((PyAtkObject *) x)->obj))
#define PAPI_ATKOBJECT_FIRSTSLOT 0
#define PAPI_ATKOBJECT_SLOTS 2
#ifndef PAPI_ATKOBJECT_INTERNAL
#define PyAtkObject_Check(x) \
    ((x)->ob_type == (PyTypeObject *) PyPapiATK_C_API[PAPI_ATKOBJECT_FIRSTSLOT])
#define PyAtkObject_New                                 \
    (*(PyObject*(*)(AtkInterfaceType))                  \
        PyPapiATK_C_API[PAPI_ATKOBJECT_FIRSTSLOT + 1])
#endif /* PAPI_ATKOBJECT_INTERNAL */

/**
 * PyObject type, that encapsulates an AtkStateSet.
 */
typedef struct
{
    PyObject_HEAD
    AtkStateSet *obj;
} PyAtkStateSet;
#define ATKSTATESET(x) (ATK_STATE_SET (((PyAtkStateSet *) x)->obj))
#define PAPI_ATKSTATESET_FIRSTSLOT \
    (PAPI_ATKOBJECT_FIRSTSLOT + PAPI_ATKOBJECT_SLOTS)
#define PAPI_ATKSTATESET_SLOTS 2
#ifndef PAPI_ATKSTATESET_INTERNAL
#define PyAtkStateSet_Check(x)                                          \
    (PyObject_TypeCheck (x,                                             \
        (PyTypeObject *) PyPapiATK_C_API[PAPI_ATKSTATESET_FIRSTSLOT]))
#define PyAtkStateSet_New \
    (*(PyObject*(*)(void))PyPapiATK_C_API[PAPI_ATKSTATESET_FIRSTSLOT + 1])
#endif /* PAPI_ATKSTATESET_INTERNAL */

/**
 * PyObject type, that encapsulates an AtkRelation.
 */
typedef struct
{
    PyObject_HEAD
    AtkRelation *obj;
} PyAtkRelation;
#define ATKRELATION(x) (ATK_RELATION (((PyAtkRelation *) x)->obj))
#define PAPI_ATKRELATION_FIRSTSLOT \
    (PAPI_ATKSTATESET_FIRSTSLOT + PAPI_ATKSTATESET_SLOTS)
#define PAPI_ATKRELATION_SLOTS 2
#ifndef PAPI_ATKRELATION_INTERNAL
#define PyAtkRelation_Check(x)                                          \
    (PyObject_TypeCheck (x,                                             \
        (PyTypeObject *) PyPapiATK_C_API[PAPI_ATKRELATION_FIRSTSLOT]))
#define PyAtkRelation_New \
    (*(PyObject*(*)(void))PyPapiATK_C_API[PAPI_ATKRELATION_FIRSTSLOT + 1])
#endif /* PAPI_ATKRELATION_INTERNAL */

/**
 * PyObject type, that encapsulates an AtkRelationSet.
 */
typedef struct
{
    PyObject_HEAD
    AtkRelationSet *obj;
} PyAtkRelationSet;
#define ATKRELATIONSET(x) (ATK_RELATION_SET (((PyAtkRelationSet *) x)->obj))
#define PAPI_ATKRELATIONSET_FIRSTSLOT \
    (PAPI_ATKRELATION_FIRSTSLOT + PAPI_ATKRELATION_SLOTS)
#define PAPI_ATKRELATIONSET_SLOTS 2
#ifndef PAPI_ATKRELATIONSET_INTERNAL
#define PyAtkRelationSet_Check(x)                                       \
    (PyObject_TypeCheck (x,                                             \
        (PyTypeObject *) PyPapiATK_C_API[PAPI_ATKRELATIONSET_FIRSTSLOT])
#define PyAtkRelationSet_New \
    (*(PyObject*(*)(void))PyPapiATK_C_API[PAPI_ATKRELATIONSET_FIRSTSLOT + 1])
#endif /* PAPI_ATKRELATIONSET_INTERNAL */

/**
 * PyObject type, that encapsulates an AtkHyperlink.
 */
typedef struct
{
    PyObject_HEAD
    AtkHyperlink *obj;
} PyAtkHyperlink;
#define ATKHYPERLINK(x) (ATK_HYPERLINK (((PyAtkHyperlink *) x)->obj))
#define PAPI_ATKHYPERLINK_FIRSTSLOT \
    (PAPI_ATKRELATIONSET_FIRSTSLOT + PAPI_ATKRELATIONSET_SLOTS)
#define PAPI_ATKHYPERLINK_SLOTS 2
#ifndef PAPI_ATKHYPERLINK_INTERNAL
#define PyAtkHyperlink_Check(x)                                         \
    (PyObject_TypeCheck (x,                                             \
        (PyTypeObject *) PyPapiATK_C_API[PAPI_ATKHYPERLINK_FIRSTSLOT]))
#define PyAtkHyperlink_New \
    (*(PyObject*(*)(void))PyPapiATK_C_API[PAPI_ATKHYPERLINK_FIRSTSLOT + 1])
#endif

/**
 * PyObject type, that encapsulates an AtkAttribute.
 */
typedef struct
{
    PyObject_HEAD
    AtkAttribute *obj;
} PyAtkAttribute;
#define ATKATTRIBUTE(x) (ATK_ATTRIBUTE (((PyAtkAttribute *) x)->obj))
#define PAPI_ATKATTRIBUTE_FIRSTSLOT \
    (PAPI_ATKHYPERLINK_FIRSTSLOT + PAPI_ATKHYPERLINK_SLOTS)
#define PAPI_ATKATTRIBUTE_SLOTS 2
#ifndef PAPI_ATKATTRIBUTE_INTERNAL
#define PyAtkAttribute_Check(x)                                         \
    (PyObject_TypeCheck (x,                                             \
        (PyTypeObject *) PyPapiATK_C_API[PAPI_ATKATTRIBUTE_FIRSTSLOT])
#define PyAtkAttribute_New \
    (*(PyObject*(*)(void))PyPapiATK_C_API[PAPI_ATKATTRIBUTE_FIRSTSLOT + 1])
#endif /* PAPI_ATKATRRIBUTE_INTERNAL */

/**
 * PyObject type, that encapsulates an AtkTextRectangle.
 */
typedef struct
{
    PyObject_HEAD
    AtkTextRectangle *obj;
} PyAtkTextRectangle;
#define ATKTEXTRECTANGLE(x) \
    ((AtkTextRectangle *) ((PyAtkTextRectangle *) x)->obj)
#define PAPI_ATKTEXTRECTANGLE_FIRSTSLOT \
    (PAPI_ATKATTRIBUTE_FIRSTSLOT + PAPI_ATKATTRIBUTE_SLOTS)
#define PAPI_ATKTEXTRECTANGLE_SLOTS 2
#ifndef PAPI_ATKTEXTRECTANGLE_INTERNAL
#define PyAtkTextRectangle_Check(x)                                     \
    (PyObject_TypeCheck (x,                                             \
        (PyTypeObject *) PyPapiATK_C_API[PAPI_ATKTEXTRECTANGLE_FIRSTSLOT])
#define PyAtkTextRectangle_New \
    (*(PyObject*(*)(void))PyPapiATK_C_API[PAPI_ATKTEXTRECTANGLE_FIRSTSLOT + 1])
#endif /* PAPI_ATKTEXTRECTANGLE_INTERNAL */

/**
 * PyObject type, that encapsulates an AtkTextRange.
 */
typedef struct
{
    PyObject_HEAD
    AtkTextRange *obj;
} PyAtkTextRange;
#define ATKTEXTRANGE(x) ((AtkTextRange *) ((PyAtkTextRange *) x)->obj)
#define PAPI_ATKTEXTRANGE_FIRSTSLOT \
    (PAPI_ATKTEXTRECTANGLE_FIRSTSLOT + PAPI_ATKTEXTRECTANGLE_SLOTS)
#define PAPI_ATKTEXTRANGE_SLOTS 2
#ifndef PAPI_ATKTEXTRANGE_INTERNAL
#define PyAtkTextRange_Check(x)                                         \
    (PyObject_TypeCheck (x,                                             \
        (PyTypeObject *) PyPapiATK_C_API[PAPI_ATKTEXTRANGE_FIRSTSLOT])
#define PyAtkTextRange_New \
    (*(PyObject*(*)(void))PyPapiATK_C_API[PAPI_ATKTEXTRANGE_FIRSTSLOT + 1])
#endif /* PAPI_ATKTEXTRANGE_INTERNAL */

static void **PyPapiATK_C_API;
#define PAPI_ATKAPI_SLOTS \
    (PAPI_ATKTEXTRANGE_FIRSTSLOT + PAPI_ATKTEXTRANGE_SLOTS)
#define PAPI_ATKCAPI_ENTRY "_PAPI_ATK_C_API"

static int
import_atk (void)
{
    PyObject *_module = PyImport_ImportModule ("papi.atk");
    if (_module != NULL)
    {
        PyObject *_capi = PyObject_GetAttrString(_module, PAPI_ATKCAPI_ENTRY);
        if (!PyCObject_Check (_capi))
        {
            Py_DECREF (_module);
            return -1;
        }
        PyPapiATK_C_API = (void**) PyCObject_AsVoidPtr (_capi);
        Py_DECREF (_capi);
        return 0;
    }
    return -1;
}

/**
 * Python module initialization function. This will be called upon the
 * import <module> statement and sets up the papi module.
 */
PyMODINIT_FUNC initatk (void);

#ifdef __cplusplus
}
#endif

#endif /* _PAPI_ATK_H_ */
